#include <cassert>

#include <cstdlib>
using std::exit;
using std::atof;
using std::atoi;

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <cmath>
using std::fabs;

bool nearly_equal(double x, double y)
{
	const double separation = fabs(x-y);
	const double eps = 1E-14;
	return separation < fabs(x)*eps and separation < fabs(y)*eps;
}

void bad()
{
	cerr << "Returns the rational power a^(p/q) and compares to the\n"
			"value computed with std::pow in the cmath library\n"
	        "Usage: ratpow a p q\n"
	        "        a=nonnegative fp number\n"
	        "      p,q=nonzero integers" << endl;
	exit(1);
}

double raise(double x, int k)
{
	if (k < 0) return 1.0/raise(x,-k);
	if (k == 0) return 1.0;
	if (k == 1) return x;
	double xk = x*x;
	while (--k > 1) { xk *= x; }
	return xk;
}

int main(int argc, char* argv[])
{
	assert(raise(2.0,0) == 1.0);
	assert(raise(2.0,1) == 2.0);
	assert(raise(2.0,2) == 4.0);
	assert(raise(2.0,3) == 8.0);
	assert(raise(2.0,-3) == 0.125);

	if (argc != 4) bad();
	const double a = atof(argv[1]);
	const int p = atoi(argv[2]);
	const int q = atoi(argv[3]);
	if (a <= 0.0 or p ==0 or q == 0) bad();
	
	const double qq = 1.0-1.0/q;
	const double ap = raise(a,p);
	
	double x = 1.0; 
	while (true)
	{
		const double xq1 = raise(x,q-1);
		const double x_new = qq*x + ap/(xq1*q);
		if (nearly_equal(x_new,x)) break;
		x = x_new;
	}
	cout.precision(16);
	cout << "Newton's method value: " << x << endl;
	cout << " C Math library value: " << pow(a,double(p)/q) << endl;
	return 0;
}
